@@ -27,8 +27,7 @@ from facebookads.exceptions import FacebookBadObjectError
 from facebookads.api import FacebookAdsApi
 
 import collections
-import pprint
-pp = pprint.PrettyPrinter(indent=4)
+import json
 
 
 class EdgeIterator(object):
@@ -125,7 +124,7 @@ class EdgeIterator(object):
         num_added = 0
         for json_obj in response['data']:
             obj = self._target_objects_class()
-            obj._read_update(json_obj)
+            obj._set_data(json_obj)
             self._queue.append(obj)
             num_added += 1
 
@@ -169,7 +168,15 @@ class AbstractObject(collections.MutableMapping):
         return unicode(self._data)
 
     def __repr__(self):
-        return pp.pformat(self._data)
+        return "<%s> %s" % (
+            self.__class__.__name__,
+            json.dumps(
+                self._data,
+                sort_keys=True,
+                indent=4,
+                separators=(',', ': ')
+            ),
+        )
 
     @classmethod
     def get_endpoint(cls):
@@ -199,25 +206,33 @@ class AbstractObject(collections.MutableMapping):
         """
         cls._default_read_fields = fields
 
-    def _read_update(self, data):
+    def _set_data(self, data):
         """
-        An AbstractObject does not keep history so _read_update is an alias for
-        a MutableMapping's update() method. _read_update elsewhere may have a
+        An AbstractObject does not keep history so _set_data is an alias for
+        a MutableMapping's update() method. _set_data elsewhere may have a
         different behavior depending on the type of the object and how the data
         should be processed.
         """
         self.update(data)
 
-    def export_data(self):
-        """Returns a dictionary of property names mapped to their values."""
-        data = {}
-
-        for key in self:
-            if self[key] is not None:
-                data[key] = self[key]
-
+    def export_value(self, data):
+        if isinstance(data, AbstractObject):
+            data = data.export_data()
+        elif isinstance(data, dict):
+            for key, value in data.items():
+                if value is None:
+                    del data[key]
+                else:
+                    data[key] = self.export_value(value)
+        elif isinstance(data, list):
+            for i, value in enumerate(data):
+                data[i] = self.export_value(value)
         return data
 
+    def export_data(self):
+        return self.export_value(self._data)
+
+
 
 class AbstractCrudObject(AbstractObject):
     """
@@ -254,37 +269,11 @@ class AbstractCrudObject(AbstractObject):
 
     def __setitem__(self, key, value):
         """Sets an item in this CRUD object while maintaining a changelog."""
-        key = str(key)
 
-        key_already_set = key in self._data
-        if key_already_set:
-            old_data_value = self._data[key]
+        if key not in self._data or self._data[key] != value:
+            self._changes[key] = value
 
-        self._data[key] = value
-
-        if key_already_set:
-            if key in self._changes:
-                # key already set and has been a result of user change
-                if (
-                    'original' in self._changes[key] and
-                    value == self._changes['key']['original']
-                ):
-                    # value is being set back to original
-                    del self._changes['key']
-                else:
-                    # There's a new value
-                    self._changes[key]['new'] = value
-            elif value != old_data_value:
-                # key already set and not in changes (has original value)
-                self._changes[key] = {
-                    'original': old_data_value,
-                    'new': value,
-                }
-        else:
-            # There's a new value
-            self._changes[key] = {
-                'new': value,
-            }
+        super(AbstractCrudObject, self).__setitem__(key, value)
 
         return self
 
@@ -379,7 +368,7 @@ class AbstractCrudObject(AbstractObject):
         self._changes = {}
         return self
 
-    def _read_update(self, data):
+    def _set_data(self, data):
         """
         Sets object's data as if it were read from the server.
         Warning: Does not log changes.
@@ -403,8 +392,7 @@ class AbstractCrudObject(AbstractObject):
         """
         data = {}
 
-        for key in self._changes:
-            value = self._changes[key]['new']
+        for key, value in self._changes.items():
             if isinstance(value, AbstractObject):
                 data[key] = value.export_data()
             else:
@@ -458,7 +446,7 @@ class AbstractCrudObject(AbstractObject):
         """
         if self.__class__.Field.id in self:
             raise FacebookBadObjectError(
-                "This %s object was alread created." % self.__class__.__name__
+                "This %s object was already created." % self.__class__.__name__
             )
 
         params = {} if params is None else params.copy()
@@ -466,7 +454,7 @@ class AbstractCrudObject(AbstractObject):
 
         if batch is not None:
             def callback_success(response):
-                self._read_update(response.json())
+                self._set_data(response.json())
                 self._clear_history()
 
                 if success is not None:
@@ -492,7 +480,7 @@ class AbstractCrudObject(AbstractObject):
                 params=params,
                 files=files,
             )
-            self._read_update(response.json())
+            self._set_data(response.json())
 
             return self
 
@@ -539,7 +527,7 @@ class AbstractCrudObject(AbstractObject):
 
         if batch is not None:
             def callback_success(response):
-                self._read_update(response.json())
+                self._set_data(response.json())
 
                 if success is not None:
                     success(response)
@@ -562,7 +550,7 @@ class AbstractCrudObject(AbstractObject):
                 self.get_node_path(),
                 params=params,
             )
-            self._read_update(response.json())
+            self._set_data(response.json())
 
             return self
 
@@ -724,12 +712,6 @@ class AbstractCrudObject(AbstractObject):
         # if nothing found, return None
         return None
 
-    def child(self, target_object_class):
-        """
-        Returns an instance of target_object_class with self's id as parent_id.
-        """
-        return target_object_class(parent_id=self.get_id_assured())
-
 
 class CannotCreate(object):
 
@@ -1116,6 +1098,7 @@ class HasObjective(object):
         canvas_app_engagement = 'CANVAS_APP_ENGAGEMENT'
         canvas_app_installs = 'CANVAS_APP_INSTALLS'
         event_responses = 'EVENT_RESPONSES'
+        local_awareness = 'LOCAL_AWARENESS'
         mobile_app_engagement = 'MOBILE_APP_ENGAGEMENT'
         mobile_app_installs = 'MOBILE_APP_INSTALLS'
         none = 'NONE'
@@ -1195,12 +1178,14 @@ class AdSet(HasStatus, AbstractCrudObject):
         bid_type = 'bid_type'
         budget_remaining = 'budget_remaining'
         campaign_group_id = 'campaign_group_id'
+        campaign_schedule = 'campaign_schedule'
         created_time = 'created_time'
         daily_budget = 'daily_budget'
         end_time = 'end_time'
         id = 'id'
         lifetime_budget = 'lifetime_budget'
         name = 'name'
+        pacing_type = 'pacing_type'
         start_time = 'start_time'
         status = 'campaign_status'
         targeting = 'targeting'
@@ -1212,6 +1197,10 @@ class AdSet(HasStatus, AbstractCrudObject):
         cpm = 'CPM'
         multi_premium = 'multi_premium'
 
+    class PacingType(object):
+        day_parting = 'day_parting'
+        standard = 'standard'
+
     @classmethod
     def get_endpoint(cls):
         return 'adcampaigns'
@@ -1330,6 +1319,7 @@ class AdCreative(AbstractCrudObject):
         object_id = 'object_id'
         object_store_url = 'object_store_url'
         object_story_id = 'object_story_id'
+        object_story_spec = 'object_story_spec'
         object_type = 'object_type'
         object_url = 'object_url'
         preview_url = 'preview_url'
@@ -1362,7 +1352,7 @@ class AdImage(CannotUpdate, AbstractCrudObject):
     def get_node_path(self):
         return (self.get_parent_id_assured(), self.get_endpoint())
 
-    def _read_update(self, data):
+    def _set_data(self, data):
         data = list(data['images'].values())[0]
 
         for key in data:
@@ -1382,22 +1372,27 @@ class AdImage(CannotUpdate, AbstractCrudObject):
 
         return self
 
-    def remote_create_from_filename(
+    def remote_create(
         self,
-        filename,
         batch=None,
         failure=None,
+        files=None,
         params=None,
         success=None,
     ):
         """Uploads filename and creates the AdImage object from it.
 
         It has same arguments as AbstractCrudObject.remote_create except it does
-        not have a 'files' keyword argument. Instead, it has a required
-        'filename' argument.
+        not have the files argument but requires the 'filename' property to be
+        defined.
         """
+        if self[self.__class__.Field.filename] is None:
+            raise FacebookBadObjectError(
+                "AdImage required a filename to be defined."
+            )
+        filename = self[self.__class__.Field.filename]
         open_file = open(filename, 'rb')
-        return_val = self.remote_create(
+        return_val = super(AdImage, self).remote_create(
             files={filename: open_file},
             batch=batch,
             failure=failure,
@@ -1756,6 +1751,7 @@ class TargetingSearch(AbstractObject):
         locale = 'adlocale'
         major = 'adeducationmajor'
         position = 'adworkposition'
+        radius_suggestion = 'adradiussuggestion'
         region = 'adregion'
         targeting_category = 'adtargetingcategory'
         zipcode = 'adzipcode'
